use bevy_app::{App, AppLabel, Last, Plugin, SubApp};
use bevy_ecs::{
    schedule::{IntoScheduleConfigs, Schedule, ScheduleLabel, SystemSet},
    system::Commands,
};

#[derive(Debug, Hash, PartialEq, Eq, Clone, SystemSet)]
pub enum RenderSystems {
    Prepare,
    Queue,
    Render,
    Present,
}

#[derive(ScheduleLabel, Debug, Hash, PartialEq, Eq, Clone, Default)]
pub struct Render;

impl Render {
    pub fn base_schedule() -> Schedule {
        use RenderSystems::*;

        let mut schedule = Schedule::new(Self);

        schedule.configure_sets((Prepare, Queue, Render, Present).chain());

        schedule
    }
}

#[derive(Debug, Clone, Copy, Hash, PartialEq, Eq, AppLabel)]
pub struct RenderApp;

pub struct RenderPlugin;

impl Plugin for RenderPlugin {
    fn build(&self, app: &mut App) {
        let mut render_app = SubApp::new();
        render_app.update_schedule = Some(Render.intern());

        render_app.add_schedule(Render::base_schedule());

        render_app.add_systems(Render, test_prepare.in_set(RenderSystems::Prepare));
        render_app.add_systems(Render, test_queue.in_set(RenderSystems::Queue));
        render_app.add_systems(Render, test_render.in_set(RenderSystems::Render));
        render_app.add_systems(Render, test_present.in_set(RenderSystems::Present));

        app.insert_sub_app(RenderApp, render_app);
    }
}

fn test_prepare(mut commands: Commands) {
    log::info!("test_prepare");
}

fn test_queue(mut commands: Commands) {
    log::info!("test_queue");
}

fn test_render(mut commands: Commands) {
    log::info!("test_render");
}

fn test_present(mut commands: Commands) {
    log::info!("test_present");
}
